/*
 * QEstEidCommon
 *
 * Copyright (C) 2010-2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2010-2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "Common.h"

#include <Cocoa/Cocoa.h>

@interface ApplicationObjC : NSObject

- (void) appReopen:(NSAppleEventDescriptor *)event withReplyEvent:(NSAppleEventDescriptor *)replyEvent;
@end

@implementation ApplicationObjC
- (id) init
{
	if( self = [super init] )
	{
		[[NSAppleEventManager sharedAppleEventManager] setEventHandler:self
			andSelector:@selector(appReopen:withReplyEvent:)
			forEventClass:kCoreEventClass
			andEventID:kAEReopenApplication];
		// reload /System/Library/CoreServices/pbs
		// list /System/Library/CoreServices/pbs -dump_pboard
		[NSApp setServicesProvider:self];
	}
	return self;
}

- (void) dealloc
{
	[[NSAppleEventManager sharedAppleEventManager]
		removeEventHandlerForEventClass:kCoreEventClass
		andEventID:kAEReopenApplication];
}

- (void) appReopen:(NSAppleEventDescriptor *)event withReplyEvent:(NSAppleEventDescriptor *)replyEvent
{
	Q_UNUSED(event)
	Q_UNUSED(replyEvent)
	QApplication::postEvent( qApp, new REOpenEvent );
}

- (void)openFile:(NSPasteboard *)pboard userData:(NSString *)data error:(NSString **)error
{
	Q_UNUSED(data)
	Q_UNUSED(error)
	NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];
	if( !files )
		return;
	NSEnumerator *enumerator = [files objectEnumerator];
	NSString *filename;
	QStringList result;
	while( (filename = [enumerator nextObject]) )
	{
		NSRange range = { 0, [filename length] };
		QString file( range.length, 0 );
		unichar *chars = reinterpret_cast<unichar *>(file.data());
		[filename getCharacters:chars range:range];
		result << file;
	}
	QApplication::postEvent( qApp, new OpenFilesEvent( result ) );
}
@end

struct ApplicationStruct { ApplicationObjC *applicationObjC; };

void Common::initMacEvents()
{
	if( macEvents )
		return;
	macEvents = new ApplicationStruct;
	macEvents->applicationObjC = [[ApplicationObjC alloc] init];
}

void Common::deinitMacEvents()
{
	if( !macEvents )
		return;
	delete macEvents;
}
